from org.python.modules import re

jvm = None
jvmArgs = ''
msOnly = 0
DCEnableProperty = "am.itcam.enabled"

serverIds = []

currentServer = ''

def resolvePath(path) :
    result = path
    while 'True' :
        match = re.search('\${([^}]+)', result)
        if match != None :
            varName = match.group(1)
            varVal = getVal(varName)
            result = re.sub('\$\{'+varName+'\}', varVal, result)
        else :
            return result

def getVal (varName) :
    global currentServer

    variables = re.split('\n|\r\n', AdminConfig.list('VariableSubstitutionEntry', currentServer))
    for var in variables :
        name = AdminConfig.showAttribute(var, 'symbolicName')
        if name == varName :
            return AdminConfig.showAttribute(var, 'value')
    return ''


def getSystemProperty(name) :
    global jvm

    props = re.split('\[|\]| ', AdminConfig.showAttribute(jvm, 'systemProperties'))
    for prop in props :
        if prop != '' :
            aname = AdminConfig.showAttribute(prop, 'name')
            if aname == name :
                return AdminConfig.showAttribute(prop, 'value')
    return None

def getDCPropertiesFile() :
    platform = getSystemProperty('appserver.platform')
    nodename = getSystemProperty('am.nodename')
    appserver = getSystemProperty('am.appserver')
    pref = ''
    if platform != None and platform != '' :
        pref = pref + platform
    if nodename != None and nodename != '' :
        pref = pref + '.' + nodename
    if appserver != None and appserver != '' :
        pref = pref + '.' + appserver

    return resolvePath('${ITCAM61HOME}/runtime/') + pref + '/' + pref + '.datacollector.properties'


def disableMSDC (server) :
    dcProp = getDCPropertiesFile()
    print 'Updating file ' + dcProp
    f = open(dcProp, 'rt')
    lines = f.readlines()
    f.close()
    changed = 0
    for i in range(len(lines)) :
        line = lines[i].strip()
        if line.find('am.itcam.enabled') == 0 :
            m = re.match('am\.itcam\.enabled *= *([^ ]*)', line)
            if m :
                if m.group(1).upper() == 'NO' :
                    print 'MS DC was already disabled for this server.'
                    return 0
                else :
                    lines[i] = 'am.itcam.enabled=no\n'
                    changed = 1
                    break
    if not changed :
        lines.append('am.itcam.enabled=no\n')
    f = open(dcProp, 'wt')

    f.writelines(lines)
    f.close()
    print 'MS DC has been disabled for this server.'
    return 1

def isDcInstalled () :
    global jvmArgs
    return (re.match('.*-Xrunam.*', jvmArgs) != None
        or re.match('.*-agentlib:am_.*', jvmArgs) != None )

def removeXbootclasspathP() :
    print 'INFO: Removing DC jars from bootclasspath/prepend'
    global jvmArgs
    while re.search('-Xbootclasspath/p:[^ ]*\${ITCAM61HOME}', jvmArgs) != None :
        jvmArgs = re.sub('(.*-Xbootclasspath/p:.*?)\$\{ITCAM61HOME\}[^;: ]+[:;]?(.*)', '\\1\\2', jvmArgs)

    if re.search('[ |]?-Xbootclasspath/p:[ |]+', jvmArgs) != None :
        jvmArgs = re.sub('-Xbootclasspath/p:[ |]+', '', jvmArgs)
    else :
        jvmArgs = re.sub('[;:] ', ' ', jvmArgs)

def removeXbootclasspathA() :
    print 'INFO: Removing RTT MA jars from bootclasspath/append'
    global jvmArgs
    while re.search('-Xbootclasspath/a:[^ ]*\${MA_INSTRUMENT', jvmArgs) != None :
        jvmArgs = re.sub('(.*-Xbootclasspath/a:.*?)\${MA_INSTRUMENT[^;: ]+[:;]?(.*)', '\\1\\2', jvmArgs)
    jvmArgs = re.sub('[;:] ', ' ', jvmArgs)

    if re.search('[ |]?-Xbootclasspath/a:[ |]+', jvmArgs) != None :
        jvmArgs = re.sub('-Xbootclasspath/a:[ |]+', '', jvmArgs)
    else :
        jvmArgs = re.sub('[;:] ', ' ', jvmArgs)

def removeXrun() :
    print 'INFO: Removing -Xrun arguments from JVM arguments'
    global jvmArgs
    jvmArgs = re.sub('-Xrunam_[^ ]+[ ]?', '', jvmArgs)
    jvmArgs = re.sub('-Xrunjvmpi[^ ]+[ ]?', '', jvmArgs)
    jvmArgs = re.sub('-agentlib:am_[^ ]+[ ]?', '', jvmArgs)

def removeSystemProperties() :
    print 'INFO: Renaming system properties'
    global jvm
    props = re.split('\[|\]| ', AdminConfig.showAttribute(jvm, 'systemProperties'))
    for prop in props :
        if prop != '' :
            name = AdminConfig.showAttribute(prop, 'name')
            if name.startswith('java.') :
                name = 'itcam.backup.' + name
                AdminConfig.modify(prop, [['name', name]])

def disableService(server) :
    print 'INFO: Disabling am Custom Service'
    services = re.split('\[|\]| ', AdminConfig.showAttribute(server, 'customServices'))
    for service in services :
        if service != '' :
            cfgUrl = AdminConfig.showAttribute(service, 'externalConfigURL')
            if cfgUrl == 'am' :
                AdminConfig.modify(service, [['enable', 'false']])

def backupJvmArgs(jvmArgs) :
    print 'INFO: Backup JVM arguments: ' + jvmArgs
    global jvm
    AdminConfig.create('Property', jvm, [['name', 'itcam.backup.jvmargs'], ['value', jvmArgs.replace('\\\\', '\\')]], 'systemProperties')

def disableDC (server) :
    global jvmArgs
    global jvm
    global msOnly

    jvm = AdminConfig.list('JavaVirtualMachine', server)
    if jvm == None or jvm == '' :
        return 0

    servertype = AdminConfig.showAttribute(server, 'serverType')
    if servertype in ['NODE_AGENT', 'DEPLOYMENT_MANAGER'] :
        print 'Skipping the ' + servertype
        return 0

    jvmArgs = AdminConfig.showAttribute(jvm, 'genericJvmArguments')
    if jvmArgs == None or jvmArgs == '' :
        return 0
    oldJvmArgs = jvmArgs
    print 'INFO: Going to disable server: ' + server
    if isDcInstalled () :
        if msOnly :
            return disableMSDC(server)

        print 'INFO: DC is installed. Disabling it'
        removeXbootclasspathP()
        removeXbootclasspathA()
        removeXrun()
        disableService(server)
        removeSystemProperties()
        backupJvmArgs(oldJvmArgs)
        if jvmArgs != oldJvmArgs :
            print 'INFO: Setting new JVM arguments: ' + jvmArgs
            AdminConfig.modify(jvm, [['genericJvmArguments', jvmArgs.replace('\\\\', '\\')]])
            return 1
    else :
        print 'INFO: DC is not installed for server or has been already disabled for server ' + server
        return 0

try :
    if len(sys.argv) < 1 :
        print 'Not enough arguments'
        sys.exit(3)

    restart = 0
    for arg in sys.argv :
        if arg == '-restart' :
            restart = 1
        elif arg == '-ms' :
            msOnly = 1

    if restart :
        sys.argv.remove('-restart')

    if msOnly :
        sys.argv.remove('-ms')
        print 'Only MS Data Collectors will be disabled'

    nodeName = sys.argv[0]
    node = AdminConfig.getid('/Node:' + nodeName)
    if node == '' :
        print 'Invalid node name - ' + nodeName
        sys.exit(2)

    serversCount = len(sys.argv) - 1
    if serversCount > 0 :
        # servers are specified manually
        i = 0
        while i < serversCount :
            serverIds.append(AdminConfig.getid('/Node:' + nodeName + ' /Server:' + sys.argv[i+1]))
            i = i + 1
    else :
        # allServers
        serverIds = re.split('\r\n|\n', AdminConfig.list('Server'))

    serversToRestart = []
    for server in serverIds :
        if server != '' :
            try :
                currentServer = server
                if disableDC (server) :
                    serversToRestart.append(server)
            except Exception :
                print 'An error occured while disabling the server '+server
            print

    if not msOnly :
        AdminConfig.save()

    if restart :
        connected = 0
        from com.ibm.ws.scripting import ScriptingException
        try :
            AdminControl.getType()
            connected = 1
        except ScriptingException :
            connected = 0

    connected = 0

    # check if conntype is not NONE
    from com.ibm.ws.scripting import ScriptingException
    try :
        AdminControl.getType()
        connected = 1
    except ScriptingException :
        connected = 0

    # get cell type (ND or no)
    cell = AdminConfig.getid('/Cell:/')
    cellType = AdminConfig.showAttribute(cell, 'cellType')

    # try to sync node if ND and conntype is not NONE
    isSync = 0

    if connected and (cellType == 'DISTRIBUTED') :
        nodeSync = AdminControl.completeObjectName('type=NodeSync,node=' + nodeName + ',*')
        try :
            print 'Syncing node ' + nodeName
            AdminControl.invoke(nodeSync, 'sync')
            isSync = 1
            print 'Syncing for node ' + nodeName + ' successfull'
        except Exception :
            isSync = 0

    # try to restart servers if needed

    if restart :
        if cellType != 'DISTRIBUTED' :
            print 'Can\'t perform restart on non-ND servers'
        elif not connected :
            print 'Since wsadmin was launched in local mode, it is impossible to restart servers'
        elif not isSync :
            print 'An attempt to sync failed. Try to sync and restart servers manually'
        else :
            runningServers = re.split('\r\n|\n', AdminControl.queryNames('node='+nodeName+',type=Server,*'))
            runningServersNames = []
            for server in runningServers :
                name = AdminControl.getAttribute(server, 'name')
                runningServersNames.append(name)

            for server in serversToRestart :
                serverName = AdminConfig.showAttribute(server, 'name')
                if serverName in runningServersNames :
                    print 'Stopping server ' + serverName
                    AdminControl.stopServer(serverName, nodeName)
                print 'Starting server ' + serverName
                AdminControl.startServer(serverName, nodeName)
                print 'Server ' + serverName + ' restarted'


except Exception :
    print 'An error occured'
    sys.exit(1)
else :
    print 'DC disabled for specified servers'
