from org.python.modules import re
jvm = None
jvmArgs = ''
props = None
temaOnly = 0
serverIds = []

currentServer = ''

def resolvePath(path) :
    result = path
    while 'True' :
        match = re.search('\${([^}]+)', result)
        if match != None :
            varName = match.group(1)
            varVal = getVal(varName)
            result = re.sub('\$\{'+varName+'\}', varVal, result)
        else :
            return result

def getVal (varName) :
    global currentServer

    variables = re.split('\n|\r\n', AdminConfig.list('VariableSubstitutionEntry', currentServer))
    for var in variables :
        name = AdminConfig.showAttribute(var, 'symbolicName')
        if name == varName :
            return AdminConfig.showAttribute(var, 'value')
    return ''


def getSystemProperty(name) :
    global jvm

    props = re.split('\[|\]| ', AdminConfig.showAttribute(jvm, 'systemProperties'))
    for prop in props :
        if prop != '' :
            aname = AdminConfig.showAttribute(prop, 'name')
            if aname == name :
                return AdminConfig.showAttribute(prop, 'value')
    return None

def getDCPropertiesFile() :
    platform = getSystemProperty('appserver.platform')
    nodename = getSystemProperty('am.nodename')
    appserver = getSystemProperty('am.appserver')
    pref = ''
    if platform != None and platform != '' :
        pref = pref + platform
    if nodename != None and nodename != '' :
        pref = pref + '.' + nodename
    if appserver != None and appserver != '' :
        pref = pref + '.' + appserver

    return resolvePath('${ITCAM61HOME}/runtime/') + pref + '/' + pref + '.datacollector.properties'


def enableMSDC (server) :
    if getVal('ITCAM61HOME') == '' :
        print 'No ITCAM 6.1 installed'
        return 0

    dcProp = getDCPropertiesFile()
    print 'Updating file ' + dcProp
    f = open(dcProp, 'rt')
    lines = f.readlines()
    f.close()
    changed = 0
    for i in range(len(lines)) :
        line = lines[i].strip()
        if line.find('am.itcam.enabled') == 0 :
            m = re.match('am\.itcam\.enabled *= *([^ ]*)', line)
            if m :
                if m.group(1).upper() != 'NO' :
                    print 'MS DC is enabled for this server.'
                    return 0
                else :
                    lines[i] = 'am.itcam.enabled=true\n'
                    changed = 1
                    break
    if not changed :
        lines.append('am.itcam.enabled=true\n')
    f = open(dcProp, 'wt')

    f.writelines(lines)
    f.close()
    print 'MS DC has been enabled for the server.'
    return 1

def disableMSDC (server) :
    if getVal('ITCAM61HOME') == '' :
        print 'No ITCAM 6.1 installed'
        return 0

    dcProp = getDCPropertiesFile()
    print 'Updating file ' + dcProp
    f = open(dcProp, 'rt')
    lines = f.readlines()
    f.close()
    changed = 0
    for i in range(len(lines)) :
        line = lines[i].strip()
        if line.find('am.itcam.enabled') == 0 :
            m = re.match('am\.itcam\.enabled *= *([^ ]*)', line)
            if m :
                if m.group(1).upper() == 'NO' :
                    print 'MS DC was already disabled for this server.'
                    return 0
                else :
                    lines[i] = 'am.itcam.enabled=no\n'
                    changed = 1
                    break
    if not changed :
        lines.append('am.itcam.enabled=no\n')
    f = open(dcProp, 'wt')

    f.writelines(lines)
    f.close()
    print 'MS DC has been disabled for this server.'
    return 1

def isDcDisabled () :
    global jvmArgs
    global props

    props = re.split('\[|\]| ', AdminConfig.showAttribute(jvm, 'systemProperties'))
    for prop in props :
        if prop != '' :
            name = AdminConfig.showAttribute(prop, 'name')
            if name == 'itcam.backup.jvmargs' :
                jvmArgs = AdminConfig.showAttribute(prop, 'value')
                props.remove(prop)
                AdminConfig.remove(prop)
                return 1
    return 0

def restoreJvmArgs () :
    global jvmArgs
    print 'INFO: Restoring JVM arguments: ' + jvmArgs
    AdminConfig.modify(jvm, [['genericJvmArguments', jvmArgs.replace('\\\\', '\\')]])

def enableService (server) :
    print 'INFO: Enabling am Custom Service'
    services = re.split('\[|\]| ', AdminConfig.showAttribute(server, 'customServices'))
    for service in services :
        if service != '' :
            cfgUrl = AdminConfig.showAttribute(service, 'externalConfigURL')
            if cfgUrl == 'am' :
                AdminConfig.modify(service, [['enable', 'true']])

def restoreSystemProperties () :
    print 'INFO: Restoring system properties'
    global props

    if props != None :
        for prop in props :
            if prop != '' :
                name = AdminConfig.showAttribute(prop, 'name')
                if name.startswith('itcam.backup.') :
                    name = name.replace ('itcam.backup.', '')
                    AdminConfig.modify(prop, [['name', name]])

def enableDC (server) :
    global jvm
    jvm = AdminConfig.list('JavaVirtualMachine', server)
    if jvm == None :
        return 0

    servertype = AdminConfig.showAttribute(server, 'serverType')
    if servertype in ['NODE_AGENT', 'DEPLOYMENT_MANAGER'] :
        print 'Skipping the ' + servertype
        return 0

    print 'INFO: Going to enable DC for server ' + server
    if temaOnly :
        msdc = disableMSDC (server)
    else :
        msdc = enableMSDC (server)

    if isDcDisabled () :
        print 'INFO: Found disabled DC '
        restoreJvmArgs()
        enableService(server)
        restoreSystemProperties()
        return 1
    else :
        if not msdc :
            print 'INFO: DC was not disabled for this server'
        return msdc

try :
    if len(sys.argv) < 1 :
        print 'Not enough arguments'
        sys.exit(3)

    restart = 0
    for arg in sys.argv :
        if arg == '-restart' :
            restart = 1
        elif arg == '-tema' :
            temaOnly = 1

    if restart :
        sys.argv.remove('-restart')

    if temaOnly :
        sys.argv.remove('-tema')
        print 'Only TEMA Data Collectors will be enabled'

    nodeName = sys.argv[0]
    node = AdminConfig.getid('/Node:' + nodeName)
    if node == '' :
        print 'Invalid node name - ' + nodeName
        sys.exit(2)

    serversCount = len(sys.argv) - 1
    if serversCount > 0 :
        # servers are specified manually
        i = 0
        while i < serversCount :
            serverIds.append(AdminConfig.getid('/Node:' + nodeName + ' /Server:' + sys.argv[i+1]))
            i = i + 1
    else :
        # allServers
        serverIds = re.split('\r\n|\n', AdminConfig.list('Server'))

    serversToRestart = []
    for server in serverIds :
        if server != '' :
            try :
                currentServer = server
                if enableDC (server) :
                    serversToRestart.append(server)
            except Exception :
                print 'An error occured while enabling the server '+server
            print

    AdminConfig.save()

    connected = 0

    # check if conntype is not NONE
    from com.ibm.ws.scripting import ScriptingException
    try :
        AdminControl.getType()
        connected = 1
    except ScriptingException :
        connected = 0

    # get cell type (ND or no)
    cell = AdminConfig.getid('/Cell:/')
    cellType = AdminConfig.showAttribute(cell, 'cellType')

    # try to sync node if ND and conntype is not NONE
    isSync = 0

    if connected and (cellType == 'DISTRIBUTED') :
        nodeSync = AdminControl.completeObjectName('type=NodeSync,node=' + nodeName + ',*')
        try :
            print 'Syncing node ' + nodeName
            AdminControl.invoke(nodeSync, 'sync')
            isSync = 1
            print 'Syncing for node ' + nodeName + ' successfull'
        except Exception :
            isSync = 0

    # try to restart servers if needed

    if restart :
        if cellType != 'DISTRIBUTED' :
            print 'Can\'t perform restart on non-ND servers'
        elif not connected :
            print 'Since wsadmin was launched in local mode, it is impossible to restart servers'
        elif not isSync :
            print 'An attempt to sync failed. Try to sync and restart servers manually'
        else :
            runningServers = re.split('\r\n|\n', AdminControl.queryNames('node='+nodeName+',type=Server,*'))
            runningServersNames = []
            for server in runningServers :
                name = AdminControl.getAttribute(server, 'name')
                runningServersNames.append(name)

            for server in serversToRestart :
                serverName = AdminConfig.showAttribute(server, 'name')
                if serverName in runningServersNames :
                    print 'Stopping server ' + serverName
                    AdminControl.stopServer(serverName, nodeName)
                print 'Starting server ' + serverName
                AdminControl.startServer(serverName, nodeName)
                print 'Server ' + serverName + ' restarted'

except Exception :
    print 'An error occured'
    sys.exit(1)
else :
    print 'DC enabled for specified servers'
